'use strict'

/* ------- beamng linking ------- */
angular.module('beamng.apps')
.directive('tabbyzTachoWTemp', [function () {
  return {
    template:
    '<object style="width:100%; height:100%; box-sizing:border-box;" type="image/svg+xml" data="/ui/modules/apps/tabbyzTachoWTemp/tacho.svg"></object>',
    replace: true,
    restrict: 'EA',
    link: function (scope, element, attrs) {
        let streams = ['driveModesInfo', 'electrics', 'engineInfo', 'escInfo']
        StreamsManager.add(streams)

        scope.$on('$destroy', function () {
            StreamsManager.remove(streams)
        })

        element.on('load', function () {
            let svgDoc = element[0].contentDocument
            let svgElement = element[0]
            svgElement.style.opacity = 0

            let driveModeText = new DriveModeText(svgDoc.getElementById('driveModeText'), 70)
            let fuelGauge = new FuelGauge(svgDoc.getElementById('fuelGauge'), svgDoc.getElementById('fuelGaugeBackground'))
            let tempGauge = new FuelGauge(svgDoc.getElementById('tempGauge'), svgDoc.getElementById('tempGaugeBackground'))
            let gearText = new GearText(svgDoc.getElementById('gearText'))
            let pBrake = new ShowHideIcon(svgDoc.getElementById('pBrakeIcon'))
            let tachometerArc = new TachometerArc(svgDoc.getElementById('rpmCurve'))
            let tachometerArcMarkers = new TachometerArcMarkers(svgDoc.getElementById('rpmCurveMarkers'), 0.5)

            let absIcon = new ShowHideIcon(svgDoc.getElementById('absIcon'))
            let escIcon = new ShowHideIcon(svgDoc.getElementById('escIcon'))
            let tcIcon = new ShowHideIcon(svgDoc.getElementById('tcIcon'))

            let leftSignalIcon = svgDoc.getElementById('leftSignalIcon')
            let rightSignalIcon = svgDoc.getElementById('rightSignalIcon')
            let indicators = new Indicators(leftSignalIcon, rightSignalIcon)

            let lowBeamIcon = svgDoc.getElementById('lowBeamIcon')
            let highBeamIcon = svgDoc.getElementById('highBeamIcon')
            let lightIcons = new LightsIcons(lowBeamIcon, highBeamIcon)

            let speedText = svgDoc.getElementById('speedText')
            let speedUnitText = svgDoc.getElementById('speedUnitText')
            let speedTexts = new SpeedTexts(speedText, speedUnitText)

            scope.$on('streamsUpdate', function (event, streams) {
                let streamIsValid = false
                if (streams.electrics && streams.engineInfo && streams.engineInfo[1]) {
                    streamIsValid = true
                }

                if (!streamIsValid) {
                    svgElement.style.opacity = 0
                } else {
                    svgElement.style.opacity = 1
                    
                    // Logs all values that are sent to the app.
                    // console.log(streams)

                    let maxRpmDisplayed = streams.engineInfo[1] * (300 / 240)
                    if (!tachometerArc.hasInitialized) { tachometerArc.initialize(0, maxRpmDisplayed) }
                    if (!tachometerArcMarkers.hasInitialized) { tachometerArcMarkers.initialize(0, maxRpmDisplayed) }
                    if (!driveModeText.hasInitialized) { driveModeText.initialize(streams.driveModesInfo) }

                    indicators.update(streams.electrics.signal_L, streams.electrics.signal_R)
                    fuelGauge.update(streams.electrics.fuel)
                    let waterTempNormalized = Math.max(Math.min((streams.electrics.watertemp - 50) / 80, 1), 0)
                    tempGauge.update(waterTempNormalized)
                    gearText.update(streams.engineInfo[5])
                    lightIcons.update(streams.electrics.highbeam, streams.electrics.lowbeam)
                    pBrake.update(streams.electrics.parkingbrake)
                    speedTexts.update(streams.electrics.wheelspeed, streams.electrics.airspeed)
                    tachometerArc.update(streams.electrics.rpmTacho)

                    absIcon.update(streams.electrics.absActive)
                    escIcon.update(streams.electrics.escActive)
                    tcIcon.update(streams.electrics.tcsActive)

                    if (driveModeText.hasInitialized && driveModeText.driveModesAreAvaliable && streams.escInfo) {
                        let name = streams.driveModesInfo.currentDriveMode.name
                        let color = streams.escInfo.activeColor
                        driveModeText.update(name, color)
                    }
                }
            })
    
            scope.$on('VehicleFocusChanged', function (event, data) {
                reInitializeUiElements([driveModeText, tachometerArc, tachometerArcMarkers])
            })
        })
    }
  }
}])